@@ -15,19 +15,14 @@
  */
 package com.lmax.disruptor.dsl;
 
-import java.util.ArrayList;
-import java.util.Collection;
-import java.util.concurrent.BrokenBarrierException;
-import java.util.concurrent.CountDownLatch;
-import java.util.concurrent.Executor;
-import java.util.concurrent.atomic.AtomicReference;
-
 import com.lmax.disruptor.BatchEventProcessor;
 import com.lmax.disruptor.BlockingWaitStrategy;
 import com.lmax.disruptor.EventHandler;
+import com.lmax.disruptor.EventProcessor;
 import com.lmax.disruptor.EventTranslator;
 import com.lmax.disruptor.ExceptionHandler;
 import com.lmax.disruptor.RingBuffer;
+import com.lmax.disruptor.Sequence;
 import com.lmax.disruptor.SequenceBarrier;
 import com.lmax.disruptor.TimeoutException;
 import com.lmax.disruptor.dsl.stubs.DelayedEventHandler;
@@ -40,22 +35,27 @@ import com.lmax.disruptor.dsl.stubs.StubExecutor;
 import com.lmax.disruptor.dsl.stubs.StubPublisher;
 import com.lmax.disruptor.dsl.stubs.TestWorkHandler;
 import com.lmax.disruptor.support.TestEvent;
-
 import org.junit.After;
 import org.junit.Before;
 import org.junit.Test;
 
+import java.util.ArrayList;
+import java.util.Collection;
+import java.util.concurrent.BrokenBarrierException;
+import java.util.concurrent.CountDownLatch;
+import java.util.concurrent.Executor;
+import java.util.concurrent.atomic.AtomicReference;
+
+import static java.lang.Thread.yield;
+import static java.util.concurrent.TimeUnit.SECONDS;
 import static org.hamcrest.CoreMatchers.equalTo;
 import static org.hamcrest.CoreMatchers.is;
+import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertNotNull;
 import static org.junit.Assert.assertSame;
 import static org.junit.Assert.assertThat;
 import static org.junit.Assert.assertTrue;
 
-
-import static java.lang.Thread.yield;
-import static java.util.concurrent.TimeUnit.SECONDS;
-
 @SuppressWarnings(value = {"unchecked"})
 public class DisruptorTest
 {
@@ -343,13 +343,6 @@ public class DisruptorTest
         workHandler2.processEvent();
     }
 
-    private TestWorkHandler createTestWorkHandler()
-    {
-        final TestWorkHandler testWorkHandler = new TestWorkHandler();
-        testWorkHandlers.add(testWorkHandler);
-        return testWorkHandler;
-    }
-
     @Test
     public void shouldSupportUsingWorkerPoolAsDependency() throws Exception
     {
@@ -361,10 +354,9 @@ public class DisruptorTest
         publishEvent();
         publishEvent();
 
-        workHandler2.processEvent();
-
         assertThat(disruptor.getBarrierFor(delayedEventHandler).getCursor(), equalTo(-1L));
 
+        workHandler2.processEvent();
         workHandler1.processEvent();
 
         delayedEventHandler.processEvent();
@@ -428,7 +420,7 @@ public class DisruptorTest
     }
 
     @Test(expected = TimeoutException.class, timeout = 2000)
-    public void shouldThrowTimeoutExceptionIfShutdownDoesntCompleteNormally() throws Exception
+    public void shouldThrowTimeoutExceptionIfShutdownDoesNotCompleteNormally() throws Exception
     {
         //Given
         final DelayedEventHandler delayedEventHandler = createDelayedEventHandler();
@@ -469,6 +461,75 @@ public class DisruptorTest
         assertThat(disruptor.getRingBuffer().remainingCapacity(), is(ringBuffer.getBufferSize() - 0L));
     }
 
+    @Test
+    public void shouldAllowEventHandlerWithSuperType() throws Exception
+    {
+        final CountDownLatch latch = new CountDownLatch(2);
+        final EventHandler<Object> objectHandler = new EventHandlerStub<Object>(latch);
+
+        disruptor.handleEventsWith(objectHandler);
+
+        ensureTwoEventsProcessedAccordingToDependencies(latch);
+    }
+
+    @Test
+    public void shouldAllowChainingEventHandlersWithSuperType() throws Exception
+    {
+        final CountDownLatch latch = new CountDownLatch(2);
+        final DelayedEventHandler delayedEventHandler = createDelayedEventHandler();
+        final EventHandler<Object> objectHandler = new EventHandlerStub<Object>(latch);
+
+        disruptor.handleEventsWith(delayedEventHandler).then(objectHandler);
+
+        ensureTwoEventsProcessedAccordingToDependencies(latch, delayedEventHandler);
+    }
+
+    @Test
+    public void shouldMakeEntriesAvailableToFirstCustomProcessorsImmediately() throws Exception
+    {
+        final CountDownLatch countDownLatch = new CountDownLatch(2);
+        final EventHandler<TestEvent> eventHandler = new EventHandlerStub(countDownLatch);
+
+        disruptor.handleEventsWith(new EventProcessorFactory<TestEvent>()
+        {
+            @Override
+            public EventProcessor createEventProcessor(final RingBuffer<TestEvent> ringBuffer, final Sequence[] barrierSequences)
+            {
+                assertEquals("Should not have had any barrier sequences", 0, barrierSequences.length);
+                return new BatchEventProcessor<TestEvent>(disruptor.getRingBuffer(), ringBuffer.newBarrier(barrierSequences), eventHandler);
+            }
+        });
+
+        ensureTwoEventsProcessedAccordingToDependencies(countDownLatch);
+    }
+
+    @Test
+    public void shouldHonourDependenciesForCustomProcessors() throws Exception
+    {
+        final CountDownLatch countDownLatch = new CountDownLatch(2);
+        final EventHandler<TestEvent> eventHandler = new EventHandlerStub(countDownLatch);
+        final DelayedEventHandler delayedEventHandler = createDelayedEventHandler();
+
+        disruptor.handleEventsWith(delayedEventHandler).then(new EventProcessorFactory<TestEvent>()
+        {
+            @Override
+            public EventProcessor createEventProcessor(final RingBuffer<TestEvent> ringBuffer, final Sequence[] barrierSequences)
+            {
+                assertSame("Should have had a barrier sequence", 1, barrierSequences.length);
+                return new BatchEventProcessor<TestEvent>(disruptor.getRingBuffer(), ringBuffer.newBarrier(barrierSequences), eventHandler);
+            }
+        });
+
+        ensureTwoEventsProcessedAccordingToDependencies(countDownLatch, delayedEventHandler);
+    }
+
+    private TestWorkHandler createTestWorkHandler()
+    {
+        final TestWorkHandler testWorkHandler = new TestWorkHandler();
+        testWorkHandlers.add(testWorkHandler);
+        return testWorkHandler;
+    }
+
     private void ensureTwoEventsProcessedAccordingToDependencies(final CountDownLatch countDownLatch,
                                                                  final DelayedEventHandler... dependencies)
         throws InterruptedException, BrokenBarrierException
